/*
  CRF++ -- Yet Another CRF toolkit

  $Id: crfpp.h,v 1.5 2006/08/07 02:35:35 taku Exp $;

  Copyright(C) 2005-2006 Taku Kudo <taku@chasen.org>

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or(at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _CRFPP_CRFPP_H
#define _CRFPP_CRFPP_H

/* C interface  */
#ifdef __cplusplus
#include <cstdio>
#else
#include <stdio.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
#include <windows.h>
#  ifdef DLL_EXPORT
#    define CRFPP_DLL_EXTERN  __declspec(dllexport)
#  else
#    define CRFPP_DLL_EXTERN  __declspec(dllimport)
#  endif
#endif

#ifndef CRFPP_DLL_EXTERN
#  define CRFPP_DLL_EXTERN extern
#endif

#ifndef SWIG

  typedef struct crfpp_t crfpp_t;

  /* C interface */
  CRFPP_DLL_EXTERN crfpp_t* crfpp_new(int,  char**);
  CRFPP_DLL_EXTERN crfpp_t* crfpp_new2(const char*);
  CRFPP_DLL_EXTERN void     crfpp_destroy(crfpp_t*);
  CRFPP_DLL_EXTERN bool     crfpp_add2(crfpp_t*, size_t, const char **);
  CRFPP_DLL_EXTERN bool     crfpp_add(crfpp_t*, const char*);
  CRFPP_DLL_EXTERN size_t   crfpp_size(crfpp_t*);
  CRFPP_DLL_EXTERN size_t   crfpp_xsize(crfpp_t*);
  CRFPP_DLL_EXTERN size_t   crfpp_dsize(crfpp_t*);
  CRFPP_DLL_EXTERN const float* crfpp_weight_vector(crfpp_t*);
  CRFPP_DLL_EXTERN size_t   crfpp_result(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN size_t   crfpp_answer(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN size_t   crfpp_y(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN size_t   crfpp_ysize(crfpp_t*);
  CRFPP_DLL_EXTERN double   crfpp_prob(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_prob2(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN double   crfpp_prob3(crfpp_t*);
  CRFPP_DLL_EXTERN double   crfpp_alpha(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_beta(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_emisstion_cost(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_next_transition_cost(crfpp_t*, size_t, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_prev_transition_cost(crfpp_t*, size_t, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_best_cost(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN const int* crfpp_emittion_vector(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN const int* crfpp_next_transition_vector(crfpp_t*, size_t, size_t, size_t);
  CRFPP_DLL_EXTERN const int* crfpp_prev_transition_vector(crfpp_t*, size_t, size_t, size_t);
  CRFPP_DLL_EXTERN double   crfpp_Z(crfpp_t*);
  CRFPP_DLL_EXTERN bool     crfpp_parse(crfpp_t*);
  CRFPP_DLL_EXTERN bool     crfpp_empty(crfpp_t*);
  CRFPP_DLL_EXTERN bool     crfpp_clear(crfpp_t*);
  CRFPP_DLL_EXTERN bool     crfpp_next(crfpp_t*);
  CRFPP_DLL_EXTERN int      crfpp_test(int, char **);
  CRFPP_DLL_EXTERN int      crfpp_learn(int, char **);
  CRFPP_DLL_EXTERN const char*  crfpp_strerror(crfpp_t*);
  CRFPP_DLL_EXTERN const char*  crfpp_yname(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN const char*  crfpp_y2(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN const char*  crfpp_x(crfpp_t*, size_t, size_t);
  CRFPP_DLL_EXTERN const char** crfpp_x2(crfpp_t*, size_t);
  CRFPP_DLL_EXTERN const char*  crfpp_parse_tostr(crfpp_t*, const char*);
  CRFPP_DLL_EXTERN const char*  crfpp_parse_tostr2(crfpp_t*, const char*, size_t);
  CRFPP_DLL_EXTERN const char*  crfpp_parse_tostr3(crfpp_t*, const char*,  size_t, char *,size_t);
  CRFPP_DLL_EXTERN const char*  crfpp_tostr(crfpp_t*);
  CRFPP_DLL_EXTERN const char*  crfpp_tostr2(crfpp_t*, char *, size_t);
#endif

#ifdef __cplusplus
}
#endif

/* C++ interface */
#ifdef __cplusplus

namespace CRFPP {

  class Tagger {
  public:
#ifndef SWIG
    virtual bool open(int,  char**) = 0;
    virtual bool open(const char*) = 0;
    virtual bool add(size_t, const char **) = 0;
    virtual void close() = 0;
#endif
    virtual bool   add(const char*) = 0;
    virtual size_t size() = 0;
    virtual size_t xsize() = 0;
    virtual size_t dsize() = 0;
    virtual const float *weight_vector() = 0;
    virtual size_t result(size_t) = 0;
    virtual size_t answer(size_t) = 0;
    virtual size_t y(size_t) = 0;
    virtual const char*   y2(size_t) = 0;
    virtual const char*   yname(size_t) = 0;
    virtual const char*   x(size_t, size_t) = 0;
    virtual const char**  x(size_t) = 0;
    virtual size_t ysize() = 0;
    virtual double prob(size_t, size_t) = 0;
    virtual double prob(size_t) = 0;
    virtual double prob() = 0;
    virtual double alpha(size_t, size_t) = 0;
    virtual double beta(size_t, size_t) = 0;
    virtual double emission_cost(size_t, size_t) = 0;
    virtual double next_transition_cost(size_t, size_t, size_t) = 0;
    virtual double prev_transition_cost(size_t, size_t, size_t) = 0;
    virtual double best_cost(size_t, size_t) = 0;
    virtual const int* emission_vector(size_t, size_t) = 0;
    virtual const int* next_transition_vector(size_t, size_t, size_t) = 0;
    virtual const int* prev_transition_vector(size_t, size_t, size_t) = 0;
    virtual double Z() = 0;
    virtual bool parse() = 0;
    virtual bool empty() = 0;
    virtual bool clear() = 0;
    virtual bool next() = 0;
    virtual const char* parse(const char*) = 0;
#ifndef SWIG
    virtual const char* toString() = 0;
    virtual const char* toString(char*, size_t) = 0;
    virtual const char* parse(const char*, size_t) = 0;
    virtual const char* parse(const char*, size_t, char*, size_t) = 0;
#endif
    virtual const char* what() = 0;
    virtual ~Tagger() {};
  };

  /* factory method */
  CRFPP_DLL_EXTERN Tagger *createTagger(int, char**);
  CRFPP_DLL_EXTERN Tagger *createTagger(const char*);
  CRFPP_DLL_EXTERN const char* getTaggerError();
}

#endif
#endif
